// test.h
// Testing Framework
// Copyright (c) 2001 Tsutomu Saito All Rights Reserved
// Version 1.00

#ifndef HEADER_TEST_H
#define HEADER_TEST_H

#include "tree.h"
#include <string>

typedef void (*CTestFunc)(const char*);

class CTestCase {
public:
	virtual ~CTestCase(){}
	virtual CTestCase* Clone() = 0;
	virtual const char* TestName()const{return "";}
	virtual void RunTest(const char* caseName) = 0;
};
#define IMP_TESTCASE(Class) \
	virtual CTestCase* Clone(){return new Class(*this);}

class CTest {
public:
	typedef tree<CTestCase*> CTree;
	typedef tree<CTestCase*>::node_iterator node_iterator;
	typedef tree<CTestCase*>::child_iterator child_iterator;
	CTest() : m_tree(0) {}
	CTest(const CTest& t){*this = t;}
	CTest& operator=(const CTest& t){if (this != &t) {
		node_iterator it;
		for (it=m_tree.begin();it!=m_tree.end();++it) delete *it;
		m_tree = t.m_tree;
		for (it=m_tree.begin();it!=m_tree.end();++it) {
			if (*it) *it = (*it)->Clone();}}
		return *this;}
	~CTest(){node_iterator it;
		for (it=m_tree.begin();it!=m_tree.end();++it) {
			CTestCase* p = *it; delete p;}}
	node_iterator Root(){return m_tree.root();}
	int Size()const{return m_tree.size();}
	void Clear(){m_tree.clear();}
	node_iterator AddTest(CTestCase* c,node_iterator it)
		{return m_tree.add_node(it,c);}
	node_iterator AddTest(CTestCase* c) {return AddTest(c,Root());}
	node_iterator AddTest(CTest& t,node_iterator it) {
		node_iterator nit = m_tree.add_tree(it,t.m_tree); it = nit;
		for (;it!=m_tree.end();++it) if (*it) *it = (*it)->Clone();
		return nit;}
	node_iterator AddTest(CTest& t) {return AddTest(t,Root());}
	/*
	node_iterator MoveTest(CTest* p,node_iterator it)
		{node_iterator nit = m_tree.add_tree(it,p->m_tree); p->Clear(); return nit;}
	node_iterator MoveTest(CTest* p) {return MoveTest(p,Root());}
	*/
	int Run(const char* caseName = "");
	const char* Message()const{return m_message.c_str();}
	static void SetTest(node_iterator it,CTestCase* c) {
		if (*it != c) {delete *it; *it = c;}}
private:
	CTree		m_tree;
	string		m_message;
};

template <typename T,typename F>
class CTempTestCase : public CTestCase {
	T*		m_org;
	F		m_func;
public:
	CTempTestCase(T* p,F f) : m_org(p), m_func(f) {}
	CTempTestCase(const CTempTestCase& tc){*this = tc;}
	CTempTestCase& operator=(const CTempTestCase& t){if (this != &t) {
		m_org = new T(*t.m_org); m_func = t.m_func;} return *this;}
	virtual ~CTempTestCase(){delete m_org;}
	virtual CTestCase* Clone(){
		return new CTempTestCase(*this);}
	virtual void RunTest(const char* caseName){(m_org->*m_func)();}
};

template <typename T,typename F>
CTest::node_iterator AddTest(CTest& t,T* p,F f,CTest::node_iterator it)
{
	return t.AddTest(new CTempTestCase<T,F>(p,f),it);
}

class CTestCaseFunc : public CTestCase {
	CTestFunc		m_pf;
	string			m_name;
public:
	CTestCaseFunc(CTestFunc f,const char* p = "") : m_pf(f), m_name(p) {}
	virtual CTestCase* Clone(){return new CTestCaseFunc(*this);}
	virtual const char* TestName()const{return m_name.c_str();}
	virtual void RunTest(const char* caseName){(*m_pf)(caseName);}
};

CTest::node_iterator AddTest(CTest& t,CTestFunc f,
		CTest::node_iterator it, const char* name);


class CTestException {
	string		m_exp;
	string		m_file;
	int			m_line;
public:
	CTestException(const char* exp,const char* file,int line)
		: m_exp(exp), m_file(file), m_line(line) {}
	const char* Express()const{return m_exp.c_str();}
	const char* File()const{return m_file.c_str();}
	int Line()const{return m_line;}
};

#define TEST_ASSERT(exp) (void)((exp) || (throw(CTestException(#exp,__FILE__,__LINE__)),true))

#endif //HEADER_TEST_H