// Tree class
// Copyright (c) 2001 Tsutomu Saito All Rights Reserved

#ifndef HEADER_TREE_H
#define HEADER_TREE_H
#pragma warning(disable : 4786) // for VC++
#include <cassert>
#include <list>
#include <vector>
#include <algorithm>
using namespace std;

template <typename Node>
class tree {
	class tag;
public:
	class node_iterator;
	typedef list<tag*>	tag_list;
	typedef tag_list::difference_type difference_type;
	typedef vector<node_iterator> child_container;
	typedef child_container::iterator child_iterator;
	class node_iterator : public iterator<bidirectional_iterator_tag,Node,difference_type> {
		friend tree;
		tag_list::iterator	m_iter;
	public:
		node_iterator(){}
		node_iterator(tag_list::iterator it) : m_iter(it) {}
		const Node& operator*()const{return (*m_iter)->m_node;}
		const Node* operator->()const{return &(*m_iter)->m_node;}
		Node& operator*(){return (*m_iter)->m_node;}
		Node* operator->(){return &(*m_iter)->m_node;}
		node_iterator& operator++(){++m_iter; return *this;}
		node_iterator operator++(int){node_iterator i = *this; ++m_iter; return i;}
		node_iterator& operator--(){--m_iter; return *this;}
		node_iterator operator--(int){node_iterator i = *this; --m_iter; return i;}
		bool operator==(const node_iterator& it)const{return m_iter == it.m_iter;}
		bool operator!=(const node_iterator& it)const{return m_iter != it.m_iter;}
		node_iterator parent(){return (*m_iter)->m_parent;}
		child_iterator begin(){return (*m_iter)->m_child.begin();}
		child_iterator end(){return (*m_iter)->m_child.end();}
		size_t size()const{return (*m_iter)->m_child.size();}
		int& temp(){return (*m_iter)->m_temp;}
	};
private:
	class tag {
		friend tree;
		friend node_iterator;
		Node			m_node;
		int				m_temp;
		node_iterator	m_parent;
		child_container	m_child;
	public:
		tag() {}
		explicit tag(const Node& nd,node_iterator it = node_iterator())
			: m_node(nd), m_parent(it) {}
		Node& node(){return m_node;}
		int& temp(){return m_temp;}
	};
	bool operator==(const tree& g); // no def
public:
	explicit tree(const Node& nd = Node()) : m_list(1,new tag(nd)) {}
	tree(const tree& t){*this = t;}
	tree& operator=(const tree& t){if (this != &t) {
		clear(); delete m_list.front(); child_iterator cit;
		tag* p = t.m_list.front(); m_list.front() = new tag(*p);
		for (cit=p->m_child.begin();cit!=p->m_child.end();++cit)
		add_tree(root(),*cit);}return *this;}
	~tree(){clear(); delete m_list.front();}
	node_iterator root(){return m_list.begin();}
	node_iterator begin(){return m_list.begin();}
	node_iterator end(){return m_list.end();}
	size_t size()const{return m_list.size();}
	void clear(){while (size() > 1) {
		node_iterator it = root(); del_node(++it);}}
	node_iterator add_node(node_iterator it,const Node& nd) {
		node_iterator nit = m_list.insert(m_list.end(),new tag(nd,it));
		(*it.m_iter)->m_child.push_back(nit); return nit;}
	node_iterator add_tree(node_iterator it,node_iterator tit) {
		node_iterator nit = add_node(it,*tit); child_iterator cit;
		for (cit=tit.begin();cit!=tit.end();++cit) add_tree(nit,*cit);
		return nit;}
	node_iterator add_tree(node_iterator it,tree& t){return add_tree(it,t.root());}
	bool del_node(node_iterator it) {
		child_iterator cit; if (it == root()) return false;
		while (it.size() > 0) del_node(*it.begin());
		child_container& cc = (*it.parent().m_iter)->m_child;
		cit = find(cc.begin(),cc.end(),it);
		if (cit == cc.end()) return false; delete *it.m_iter;
		cc.erase(cit); m_list.erase(it.m_iter); return true;}
private:
	tag_list	m_list;
};

// VACYp
#ifndef TEMPL_DELIMITER
#define TEMPL_DELIMITER
template<typename T> void delimiter(T& t,char c) {}
#endif // TEMPL_DELIMITER
// fstream gꍇȉ̃RgOAK .cpp Ɉڂ
// #include <fstream>
// template<> void delimiter<ofstream>(ofstream& f,char c) {f << c;}
// template<> void delimiter<ostream>(ostream& o,char c) {o << c;}

template<typename S,typename N>
S& operator<<(S& s,tree<N>& t)
{
	tree<N>::node_iterator	nit;
	s << t.size();
	delimiter(s,'\n');
	int			i = 0;
	for (nit=t.begin();nit!=t.end();++nit) nit.temp() = i++;
	nit = t.root();
	s << *nit;
	delimiter(s,'\n');
	for (++nit;nit!=t.end();++nit) {
		s << nit.parent().temp();
		delimiter(s,' ');
		s << *nit;
		delimiter(s,'\n');
	}
	return s;
}
template<typename S,typename N>
S& operator>>(S& s,tree<N>& t)
{
	int		i,k,n;
	N		nd;
	tree<N>::node_iterator	nit;
	t.clear();
	s >> n >> *t.root();
	vector<tree<N>::node_iterator>	v(n);
	assert(n > 0);
	v[0] = t.root();
	for (i=1;i<n;++i) {
		s>> k >> nd ;
		assert(k >= 0 && k < i);
		v[i] = t.add_node(v[k],nd);
	}
	return s;
}

#endif //HEADER_TREE_H